@@ -11,11 +11,12 @@ use XML::Simple;
 use LWP::UserAgent;
 use LWP::ConnCache;
 use MIME::Entity;
+use MIME::Base64;
 use IO::Lambda qw(:all);
 use IO::Lambda::HTTP qw(http_request);
+use Crypt::OpenSSL::RSA;
 
-
-our $VERSION = '0.04';
+our $VERSION = '0.06';
 
 sub new
 {
@@ -23,12 +24,11 @@ sub new
 	my $self = bless {
 		cpr        => '0000000000',
 		password   => '',
-		activation => '',
 		country    => 'DK',
 		type       => 'P',
-		deviceid   => 'DEADBEEF-1337-1337-1337-000000000000',
 		datetime   => DateTime->now->strftime('%Y-%m-%d %H:%M:%SZ'),
 		root       => 'rest.e-boks.dk',
+		deviceid   => 'DEADBEEF-1337-1337-1337-900000000002',
 
 		nonce      => '',
 		sessionid  => '',
@@ -40,8 +40,6 @@ sub new
 		%opts,
 	}, $class;
 
-	$self->{challenge} = sha256_hex(sha256_hex(join(':', @{$self}{qw(activation deviceid type cpr country password datetime)})));
-
 	return $self;
 }
 
@@ -86,10 +84,14 @@ sub login
 
 	return undef if defined $self->{uid};
 
+	local $self->{challenge} = sha256_hex(sha256_hex(join(':', EBOKS => @{$self}{
+		qw(deviceid type cpr country password datetime)})));
 	my $authstr = 'logon ' . join(',', map { "$_=\"$self->{$_}\"" } qw(deviceid datetime challenge));
 	my $content = <<XML;
-<Logon xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns="urn:eboks:mobile:1.0.0">
-<User identity="$self->{cpr}" identityType="$self->{type}" nationality="$self->{country}" pincode="$self->{password}"/>
+<Logon xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
+xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns="urn:eboks:mobile:1.0.0">
+<User identity="$self->{cpr}" identityType="$self->{type}"
+nationality="$self->{country}" pincode="$self->{password}"/>
 </Logon>
 XML
 
@@ -120,6 +122,127 @@ XML
 	};
 }
 
+sub login_nemid
+{
+	my $self = shift;
+
+	return undef if defined $self->{uid};
+
+	# openssl genrsa -out id_rsa 2048
+	my $pk = Crypt::OpenSSL::RSA->new_private_key(<<'PVT');
+-----BEGIN RSA PRIVATE KEY-----
+MIIEpAIBAAKCAQEA2VUahnbWKIY4rn8jEthY9M2BoMIHoNQlY4YUL9pV+MpSKyy9
+MjVKV6h8ERnj+1wxUJDR3ZJimYnvcruGqlSR+uhL8MJs7GqSSOL3zKbZiHmip1/j
+/9Wzsu86VJibxd14/5r8OugIJDs+aeE6fxpKW1BtUiiUAvlbC4MwnAnCPemzl7gG
+qi64xsSaVdoi0NzZpxI+ItP9x89eMw64F5GlIviGJ9hODyW3ckKSvgxEQGf7x9TN
+toVt1Gxh4jdokalHmgNQy4zaqnzGLstl227HIEfbbzX/rK30FFVurFG0JAE9T7z7
+b0S5RkGFx4GgKGRoFRd8HE+UptBa4JyvmvA3MQIDAQABAoIBAQCtEkbDWhOFxg2R
+eJGXyk5c9OMMADhO7WKw9O9ShE7+hzAUTdaFC0ces3/JppKVc3+aJxnZl1+J4fyb
+o5bEQgDWjPMc0dgoFV5VSNoJUb3eHu9W1tgcvjQShMww3i7+zTY0Z1oCFxGUuNEl
+REVvPqKEQXItgT8Nd0H30wt815Cl9+RlaXMmNRq6aCB0GSUHpVGmgasiUejk7Zej
+rp23LarcmZitiQXGt0yCbW35/6553Ph88W4cgfav6y+LKTxK06UkKh/QRJaCoKLV
+BXVmb5HCZv5waU5eRaxV/TKTATDU45DuU3f76+OlQp0P/cH4EVXhC+95fLJ1XUxG
+lSIQW+MhAoGBAO7N0o5llwbgNpxzZ6z4mRni3LUFpURozNT3q616qM7QGBRfDeHI
+o1cZf8wRWmMBdA16iWT7xMTpRfHcHt8NW+XlzisQG5KOlrUh1ZtRrltFGPnxUd84
+EgbUK+ArzU4mqZZMETZBmrJqVO1lB0dhrjqZ9SDUeXimNqZCoGHU/Z9VAoGBAOj7
+d/bhp4OtRKBDgKF1IThikgnsZAcBfGCXaASrPFAbi1p4MNEHUNkQXG7V/PSxMlUf
+y3U35Hsxexpq2Al98gw/TUBdgb/WfDJHole1fbXTd/Gh9H8RdMLSnOdLTCMXvW9r
+e1DKt8/5fb87BE8xQUc9sXJ5mmx522WqEyOXZeBtAoGAbIUQCDHWXgOKDbLMDGi0
+enUDwyeboOjXHHiohZ9WExWxu6AumMoqoCwwTTYdkxxX9sAWq9NV6f3wESbsyIQz
+nNe/xwX84a72gb2sanbF+yf9X6fwgrXiS0Qj5C1DkR40tt4+fB94A1ga2/6rPh7/
+pBXOtWqZAODXuNpSM+MsljkCgYBWV9u9wyMxyaUFP/8L1zzYiK9WviTT89kEcxg5
+orxXc93RSXnN/cgYqdeXu/ZjOMhOg9oDNxOWFGBrCe3GlsZ9g3g9wmmzjum4OJQR
+rVFJcXWiN0NFVFLRYPyFO4Kb/tBV2p948afti6julhCiyL5IiLSamDaCvSZyJvWw
+2wsGgQKBgQCt6ityOP56Co60pUnhonmNLg98IvWnREn8xoGdjtrMuk+Ksf5sUX5i
+fcogYnJ4ciLmJ37cVXfOtrRrDsjqSbHY07Oqb0qdIKPATJkiK0ltXG4hSvjB2LPU
+XC53Clpk+n6+ltUJnUAFtl8g4jcUG9Bs+334WiX0n7Hx7yQlsvzBtA==
+-----END RSA PRIVATE KEY-----
+PVT
+	local $self->{challenge} = join(':', q(EBOKS), @{$self}{qw(deviceid type cpr country password datetime)});
+	$self->{challenge} = encode_base64($pk->sign($self->{challenge}));
+
+	my $authstr = 'logon ' . join(',', map { "$_=\"$self->{$_}\"" } qw(deviceid datetime challenge));
+	my $content = <<XML;
+<Logon xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
+xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns="urn:eboks:mobile:1.0.0">
+<App version="$VERSION" os="$^O" osVersion="1" Device="pc"/>
+<User identity="$self->{cpr}" identityType="$self->{type}"
+nationality="$self->{country}" pincode="$self->{password}"/>
+</Logon>
+XML
+
+	my $login = HTTP::Request->new(
+		'PUT',
+		'https://' . $self->{root} . '/mobile/1/xml.svc/en-gb/session',
+		[
+			'Content-Type'         => 'application/xml',
+			'Content-Length'       => length($content),
+			'X-EBOKS-AUTHENTICATE' => $authstr,
+			'Accept'               => '*/*',
+			'Accept-Language'      => 'en-US',
+			'Accept-Encoding'      => 'gzip,deflate',
+			'Host'                 => $self->{root},
+		],
+		$content
+	);
+	$login->protocol('HTTP/1.1');
+
+	return $login, sub { 
+		my ($xml, $error) = $self-> response({ForceArray => 0}, @_);
+		return $xml, $error unless $xml;
+		return undef, "'User' is not present in response" unless exists $xml->{User};
+
+		$self->{uid}   = $xml->{User}->{userId};
+		$self->{uname} = $xml->{User}->{name};
+		return $self->{uname};
+	};
+}
+
+sub session_activate
+{
+	my ($self, $ticket) = @_;
+
+# 	openssl rsa -in id_rsa -outform PEM -pubout -out id_rsa.pub
+	my $pubkey = join '', grep {!/^--/} split /\n/, <<'PUB';
+-----BEGIN PUBLIC KEY-----
+MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEA2VUahnbWKIY4rn8jEthY
+9M2BoMIHoNQlY4YUL9pV+MpSKyy9MjVKV6h8ERnj+1wxUJDR3ZJimYnvcruGqlSR
++uhL8MJs7GqSSOL3zKbZiHmip1/j/9Wzsu86VJibxd14/5r8OugIJDs+aeE6fxpK
+W1BtUiiUAvlbC4MwnAnCPemzl7gGqi64xsSaVdoi0NzZpxI+ItP9x89eMw64F5Gl
+IviGJ9hODyW3ckKSvgxEQGf7x9TNtoVt1Gxh4jdokalHmgNQy4zaqnzGLstl227H
+IEfbbzX/rK30FFVurFG0JAE9T7z7b0S5RkGFx4GgKGRoFRd8HE+UptBa4JyvmvA3
+MQIDAQAB
+-----END PUBLIC KEY-----
+PUB
+
+	my $authstr = join(',', map { "$_=\"$self->{$_}\"" } qw(deviceid nonce sessionid response));
+	my $content = <<XML;
+<?xml version="1.0" encoding="utf-8"?>
+<Activation xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
+xmlns:xsd="http://www.w3.org/2001/XMLSchema" deviceId="$self->{deviceid}"
+deviceName="pc" deviceOs="$^O" key="$pubkey" ticket="$ticket"
+ticketType="KspWeb" xmlns="urn:eboks:mobile:1.0.0" />
+XML
+
+	my $login = HTTP::Request->new(
+		'PUT',
+		'https://' . $self->{root} . "/mobile/1/xml.svc/en-gb/$self->{uid}/0/session/activate",
+		[
+			'Content-Type'         => 'application/xml',
+			'Content-Length'       => length($content),
+			'X-EBOKS-AUTHENTICATE' => $authstr,
+			'Accept'               => '*/*',
+			'Accept-Language'      => 'en-US',
+			'Accept-Encoding'      => 'gzip,deflate',
+			'Host'                 => $self->{root},
+		],
+		$content
+	);
+	$login->protocol('HTTP/1.1');
+
+	return $login, sub { $self-> response(0, @_) };
+}
+
 sub ua { LWP::UserAgent->new(conn_cache => shift->{conn_cache}) }
 
 sub get
@@ -243,6 +366,13 @@ sub first_value
 	}
 }
 
+sub safe_encode
+{
+	my ($enc, $text) = @_;
+	utf8::downgrade($text, 'fail silently please');
+	return (utf8::is_utf8($text) || $text =~ /[\x80-\xff]/) ? encode($enc, $text) : $text;
+}
+
 sub assemble_mail
 {
 	my ( $self, %opt ) = @_;
@@ -269,9 +399,9 @@ sub assemble_mail
 	$received = $date->strftime('%a, %d %b %Y %H:%M:%S %z');
 
 	my $mail = MIME::Entity->build(
-		From          => $opt{from}    // ( encode('MIME-Q', $sender) . ' <noreply@e-boks.dk>' ) ,
-		To            => $opt{to}      // ( encode('MIME-Q', $self->{uname}) . ' <' . ( $ENV{USER} // 'you' ) . '@localhost>' ),
-		Subject       => $opt{subject} // encode('MIME-Header', $msg->{name}),
+		From          => $opt{from}    // ( safe_encode('MIME-Q', $sender) . ' <noreply@e-boks.dk>' ) ,
+		To            => $opt{to}      // ( safe_encode('MIME-Q', $self->{uname}) . ' <' . ( $ENV{USER} // 'you' ) . '@localhost>' ),
+		Subject       => $opt{subject} // safe_encode('MIME-Header', $msg->{name}),
 		Data          => $opt{data}    // encode('utf-8', "Mail from $sender"),
 		Date          => $opt{date}    // $received,
 		Charset       => 'utf-8',
@@ -290,6 +420,7 @@ sub assemble_mail
 	for ( @attachments ) {
 		my ( $msg, $body ) = @$_;
 		my $fn = $self->filename($msg);
+		Encode::_utf8_off($body);
 		my $entity = $mail->attach(
 			Type     => $self->mime_type($msg),
 			Encoding => 'base64',
@@ -299,11 +430,10 @@ sub assemble_mail
 
 		# XXX hack filename for utf8
 		next unless $fn =~ m/[^\x00-\x80]/;
-		$fn = Encode::encode('utf-8', $fn);
-		$fn =~ s/([^A-Za-z])/'%'.sprintf("%02x",ord($1))/ge;
+		$fn = Encode::encode('MIME-B', $fn);
 		for ( 'Content-disposition', 'Content-type') {
 			my $v = $entity->head->get($_);
-			$v =~ s/name="(.*)"/name*=.utf-8''$fn/;
+			$v =~ s/name="(.*)"/name="$fn"/;
 			$entity->head->replace($_, $v);
 		}
 	}
@@ -382,6 +512,7 @@ sub list_all_messages
 		return ($xml, $error) unless $xml;
 
 		%ret = ( %ret, %$xml );
+		#delete $ret{0}; # generates 500 server error
 		return \%ret if keys(%$xml) < $limit;
 
 		$offset += $limit;
